/*
 * VNPForth - Compiled native Forth for IA32 Linux systems
 * Copyright (C) 2001  Simon Baldwin (simonb@caldera.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

%{
#define	YYSTYPE		char*
#include "forth.tab.h"

#include "forth_y.h"

/* Line number information for error reporting. */
int		yy_linenum = 1;

%}

/* Definition of digit and Forth 'graphics' character. */
DIGIT	[0-9]
ODIGIT	[0-7]
HDIGIT	[0-9A-F]
FCHAR	[!-~]
SPACE	[ \t]

/* Special CODE...END-CODE states. */
%x code1 code2 code3

/* Scan patterns. */
%%

<*>"\n"					yy_linenum++; REJECT;

[-+]?{DIGIT}+				yylval = yytext; return INTEGER_LITERAL;
0x{HDIGIT}+				yylval = yytext; return INTEGER_LITERAL;
0{ODIGIT}+				yylval = yytext; return INTEGER_LITERAL;

[-+]?{DIGIT}+("."{DIGIT}*)?[Ee][-+]?{DIGIT}* {
					yylval = yytext; return FLOAT_LITERAL; }

[-+]?{DIGIT}+"."{DIGIT}*		yylval = yytext; return FLOAT_LITERAL;

'{SPACE}*{FCHAR}+			yylval = yytext; return OBJECT_ADDRESS;
"["'"]"{SPACE}*{FCHAR}+			yylval = yytext; return OBJECT_ADDRESS;

S\"" "[^\"]*\"				yylval = yytext; return STRING_LITERAL;
C\"" "[^\"]*\"				yylval = yytext; return CSTRING;
"."\"" "[^\"]*\"			yylval = yytext; return PSTRING;

CHAR{SPACE}+{FCHAR}+			yylval = yytext; return CHAR_LITERAL;
\[CHAR\]{SPACE}+{FCHAR}+		yylval = yytext; return CHAR_LITERAL;

:					yylval = yytext; return ':';
;					yylval = yytext; return ';';
EXIT					yylval = yytext; return EXIT;

VARIABLE				yylval = yytext; return VARIABLE;
CREATE					yylval = yytext; return CREATE;
ALLOT					yylval = yytext; return ALLOT;
CONSTANT				yylval = yytext; return CONSTANT;

IF					yylval = yytext; return IF;
ELSE					yylval = yytext; return ELSE;
THEN					yylval = yytext; return THEN;

DO					yylval = yytext; return DO;
"?"DO					yylval = yytext; return QUERY_DO;
LOOP					yylval = yytext; return LOOP;
"+"LOOP					yylval = yytext; return PLUS_LOOP;
LEAVE					yylval = yytext; return LEAVE;

BEGIN					yylval = yytext; return _BEGIN;
UNTIL					yylval = yytext; return UNTIL;
AGAIN					yylval = yytext; return AGAIN;
WHILE					yylval = yytext; return WHILE;
REPEAT					yylval = yytext; return REPEAT;

CASE					yylval = yytext; return CASE;
OF					yylval = yytext; return OF;
ENDOF					yylval = yytext; return ENDOF;
ENDCASE					yylval = yytext; return ENDCASE;

RECURSE					yylval = yytext; return RECURSE;

CODE					{ BEGIN(code1);
					  yylval = yytext; return CODE; }
<code1>{
	{SPACE}+			/* Ignore whitespace. */
	{FCHAR}+			{ BEGIN(code2);
					  yylval = yytext; return WORD; }
}
<code2>{
	{SPACE}+			/* Ignore whitespace. */
	"\\"{SPACE}[^\n]*\n		/* Ignore \ comments. */ {
					  BEGIN(code3);
					  yy_linenum++; }
	"\\"\n				/* Ignore \<cr> comments. */ {
					  BEGIN(code3);
					  yy_linenum++; }
	"("{SPACE}[^)\n]*")"		/* Ignore (...) comments. */
	"\n"				  BEGIN(code3);
	.				{ BEGIN(INITIAL);
					  yyerror ("expected newline here"); }
}
<code3>{
	"\n"				/* Ignore newline. */
	END-CODE			{ BEGIN(INITIAL);
					  yylval = yytext; return END_CODE; }
	[^\n]+				  yylval = yytext; return ASM;
}
<code1,code2,code3>.			{ BEGIN(INITIAL);
					  yyerrora ("unknown character '%s'",
								yytext); }
<code1,code2,code3><<EOF>>		{ BEGIN(INITIAL);
					  yyerrora
					    ("unexpected %s in CODE...ENDCODE",
								"EOF");
					  yyterminate (); }

{SPACE}+				/* Ignore whitespace. */
"\\"{SPACE}[^\n]*\n			/* Ignore \ comments. */ {
					  yy_linenum++; }
"\\"\n					/* Ignore \<cr> comments. */ {
					  yy_linenum++; }
"("{SPACE}[^)\n]*")"			/* Ignore (...) comments. */
"\n"					/* Ignore newlines. */

{FCHAR}+				yylval = yytext; return WORD;

.					{ yyerrora ("unknown character '%s'",
								yytext); }

%%

