/*
 * VNPForth - Compiled native Forth for IA32 Linux systems
 * Copyright (C) 2001  Simon Baldwin (simonb@caldera.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <assert.h>
#include <stdio.h>
#include <sys/param.h>
#include <errno.h>

#include "ifile.h"

/* File stream, etc. */
static FILE	*fs			= NULL;
static char	fname[MAXPATHLEN]	= "";


/*
 * close_ifile()
 *
 * Close the current ifile
 */
void
close_ifile (void)
{
	/* Do nothing if no file is open. */
	if (fs != NULL)
	    {
		/* Reset pointers and whatnot. */
		fclose (fs);
		fs = NULL;
	    }
}


/*
 * open_ifile()
 *
 * Open a new ifile.
 */
int
open_ifile (const char *path)
{
	assert (path != NULL && strlen (path) < sizeof (fname) - 1);

	/* Close any already-open file, then open the new one. */
	close_ifile ();
	fs = fopen (path, "r");
	if (fs == NULL)
		return errno;

	/* Note the path to the opened file. */
	strcpy (fname, path);
	return 0;
}


/*
 * get_ifname()
 *
 * Get the name of the file opened, or last opened.
 */
char *
get_ifname (void)
{
	return fname;
}

/*
 * get_ifstream()
 *
 * Get the file stream of the file opened, NULL if not open.
 */
FILE *
get_ifstream (void)
{
	return fs;
}
