/*
 * VNPForth - Compiled native Forth for IA32 Linux systems
 * Copyright (C) 2001  Simon Baldwin (simonb@caldera.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef _PSTORE_H
#define _PSTORE_H

/* Definitions of symbol and data table structures. */
enum elunits { P_CELL, P_CHAR, P_UNKNOWN };
enum symtype { P_VARIABLE, P_CONSTANT, P_EXTERNALWORD, P_WORD, P_CODEWORD };
struct symrec {
	unsigned int	magic;		/* Magic number identifier */
	int		id;		/* Symbol unique id */
	enum symtype	type;		/* Symbol type */
	char		*name;		/* Symbol name */
	int		len;		/* Length of variable */
	enum elunits	units;		/* Units of length of variable */
	int		linenum;	/* Debug: source line number */
	struct symrec	*next;		/* Private: pointer to next in list */
	int		hash;		/* Private: hash code for this name */
	struct symrec	*next_hash;	/* Private: next in hash list */
};
enum datatype { P_DATA, P_STRING, P_COUNTEDSTRING };
struct datarec {
	unsigned int	magic;		/* Magic number identifier */
	int		id;		/* Data unique id */
	enum datatype	type;		/* Data type */
	void		*data;		/* Pointer to data */
	int		len;		/* Length of data, in bytes */
	int		linenum;	/* Debug: source line number */
	struct datarec	*next;		/* Private: Pointer to next in list */
};

/* Definition of the pcode structure. */
struct pcoderec {
	unsigned int	magic;		/* Magic number identifier */
	char		*opcode;	/* Opcode */
	char		*operands;	/* Operands, may be NULL */
	char		*source_token;	/* Debug: source token, may be NULL */
	int		linenum;	/* Debug: source line number */
	struct pcoderec	*next;		/* Private: pointer to next in list */
};

/* Externally available functions. */
extern struct symrec	*p_addsym (enum symtype type, const char *symname,
						int len, enum elunits units,
						int linenum);
extern void		p_chgsym (struct symrec *symp, enum symtype type,
						int len, enum elunits units);
extern struct symrec	*p_findsym (const char *symname);
extern struct datarec	*p_adddata (enum datatype type, void *data, int len,
						int linenum);
extern void		p_append (const char *opcode, const char *operands,
						const char *source_token,
						int source_linenum);
extern void		p_replace (struct pcoderec *pcodep, const char *opcode,
						const char *operands);
extern void		p_printout (FILE *stream);
extern void		p_empty (void);
extern struct datarec	*p_lookupdata (int dataid);
extern struct symrec	*p_lookupsym (int symid);
extern struct pcoderec	*p_codestart (void);
extern struct pcoderec	*p_codenext (struct pcoderec *current);
extern struct symrec	*p_symstart (void);
extern struct symrec	*p_symnext (struct symrec *current);
extern struct datarec	*p_datastart (void);
extern struct datarec	*p_datanext (struct datarec *current);

#endif
